﻿using SE.GOV.MM.Integration.Outlook.BusinessLayer.BusinessObject;
using SE.GOV.MM.Integration.Outlook.BusinessLayer.Helper;
using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace SE.GOV.MM.Integration.Outlook
{
    public partial class CheckStatusForm : Form
    {
        public string Sender { get; set; }

        public CheckStatusForm(string sender)
        {
            InitializeComponent();
            Sender = sender;
            this.KeyDown += CheckStatusForm_KeyDown;
            this.KeyPreview = true;
        }

        /// <summary>
        /// Escape closes the form.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CheckStatusForm_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
            {
                Close();
            }
        }

        /// <summary>
        /// Shows form as a dialog.
        /// </summary>
        public void ShowCheckStatusForm()
        {
            this.ShowDialog();
        }

        /// <summary>
        /// Event for searching latest sent messages 
        /// </summary>
        private void btnCheckLatest_Click(object sender, EventArgs e)
        {
            dgvStatus.Rows.Clear();
            try
            {
                if (ConfigHelper.ConfigurationEntity != null)
                {
                    var deliveryresult = new MessageDeliveryResult(Sender);

                    var result = deliveryresult.GetDeliveryStatusResult(string.Empty);
                    if (result != null)
                    {
                        fillGridWithDeliveryResult(result);
                    }
                    else
                    {
                        MessageBox.Show(ResourceHelper.Resources[10].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show(ResourceHelper.Resources[10].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception)
            {
                MessageBox.Show(ResourceHelper.Resources[10].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }           
        }

        /// <summary>
        /// Event for searching on a specific recipient 
        /// </summary>
        private void btn_CheckRecipient_Click(object sender, EventArgs e)
        {
            dgvStatus.Rows.Clear();
            try
            {
                if (!string.IsNullOrEmpty(tbxRecipient.Text))
                {
                    if (new RecipientHelper().CheckInvalidChars(tbxRecipient.Text))
                    {
                        if (ConfigHelper.ConfigurationEntity != null)
                        {
                            var rHelper = new RecipientHelper();
                            // Add century number to the recipient, 8101012386 -> 198101012386
                            var recipient = rHelper.AddCenturyNumbers(tbxRecipient.Text.Trim());
                            tbxRecipient.Text = recipient;

                            if (rHelper.ValidRecipient(recipient))
                            {
                                recipient = rHelper.GetRecipientAdress(recipient);
                                var deliveryresult = new MessageDeliveryResult(Sender);
                                var result = deliveryresult.GetDeliveryStatusResult(recipient);
                                if (result != null)
                                {
                                    fillGridWithDeliveryResult(result);
                                }
                            }
                            else
                            {
                                MessageBox.Show(ResourceHelper.Resources[3].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                            }
                        }
                        else
                        {
                            MessageBox.Show(ResourceHelper.Resources[10].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("Ogiltigt tecken för personnummer alt. organisationsnummer. Kontrollera och försök igen.", ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else
                {
                    MessageBox.Show(ResourceHelper.Resources[3].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception)
            {
                MessageBox.Show(ResourceHelper.Resources[10].Text, ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }    
        }

        /// <summary>
        /// Fill grid dgvStatus with deliverystatus
        /// </summary>
        /// <param name="result"></param>
        private void fillGridWithDeliveryResult(List<DeliveryResult> result)
        {
            if (result.Count > 0)
            {
                foreach (DeliveryResult dResult in result)
                {
                    var rowIndex = dgvStatus.Rows.Add();
                    dgvStatus.Rows[rowIndex].Cells[0].Value = dResult.Recipient;
                    dgvStatus.Rows[rowIndex].Cells[1].Value = dResult.CreatedDate;

                    switch (dResult.Status)
                    {
                        case DeliveryStatus.Delivered:
                            dgvStatus.Rows[rowIndex].Cells[2].Value = "Levererat";
                            break;
                        case DeliveryStatus.Failed:
                            dgvStatus.Rows[rowIndex].Cells[2].Value = "Fel";
                            break;
                        case DeliveryStatus.Pending:
                            dgvStatus.Rows[rowIndex].Cells[2].Value = "Pågår";
                            break;
                        default:
                            dgvStatus.Rows[rowIndex].Cells[2].Value = "Okänt";
                            break;
                    }

                    dgvStatus.Rows[rowIndex].Cells[3].Value = dResult.DeliveredDate;
                }
            }
            else
            {
                MessageBox.Show("Du har inte skickat några meddelanden till mottagaren.", ResourceHelper.Resources[1].Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }
    }
}
