﻿using SE.GOV.MM.Integration.Package.DataTransferObjects.BusinessObjects;
using SE.GOV.MM.Integration.FaR.DataTransferObjects.BusinessObject;
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using SE.GOV.MM.Integration.Outlook.BusinessLayer.Helper;

namespace SE.GOV.MM.Integration.Outlook.Helper
{
    public class AttachmentHelper
    {
        public AttachmentHelper() { }

        /// <summary>
        /// Get filter for showfiledialog in gui.
        /// "Mina meddelanden |*.pdf;*.doc;*.docx;*.ics";
        /// </summary>
        public string GetAttachmentFilter()
        {
            var sBuilder = new StringBuilder("Mina meddelanden |");

            for (int i = 0; i < ConfigHelper.ConfigurationEntity.AttachmentExtensions.Count; i++)
            {
                sBuilder.Append(string.Format("*{0};", ConfigHelper.ConfigurationEntity.AttachmentExtensions[i].Extension));
            }

            return sBuilder.ToString();
        }

        /// <summary>
        /// Creates a attachment object.
        /// </summary>
        public Attachment GetAttachment(FileInfo attachment)
        {
            return getAttachment(attachment);
        }

        /// <summary>
        /// Checks if Attachments are allowed to be sent 
        /// </summary>
        public bool ValidateAttachment(List<Attachment> attachments)
        {
            var maxSize = ConfigHelper.ConfigurationEntity.MaxTotalAttachmentSize;
            var totalSize = 0;

            foreach (Attachment attachment in attachments)
            {
                totalSize += attachment.Size;
                if (totalSize > maxSize)
                {
                    throw new ArgumentException(ResourceHelper.Resources[26].Text);
                }

                if (!checkAttachmentExtension(attachment.Filename, ConfigHelper.ConfigurationEntity.AttachmentExtensions))
                {
                    throw new ArgumentException(ResourceHelper.Resources[25].Text);
                }
            }

            return true;
        }

        /// <summary>
        /// Check if file extension is valid as an attachment.
        /// </summary>
        private bool checkAttachmentExtension(string filename, List<AttachmentExtension> allowed)
        {
           //Create substring of extension.
           string extension = getAttachmentExtension(filename);
           foreach (AttachmentExtension attachmentExtension in allowed)
           {
               if (attachmentExtension.Extension == extension)
                   return true;
           }
           return false;
        }

        /// <summary>
        /// Gets a attachments file extension
        /// </summary>
        private string getAttachmentExtension(string fileName)
        {
            char[] splitter = new char[] { '.' };
            string[] splittedFilename = fileName.Split(splitter);
            string extension = string.Format(".{0}", splittedFilename[splittedFilename.Length -1]);
            return extension;
        }

        /// <summary>
        /// Create a new Attachment
        /// </summary>
        private Attachment getAttachment(FileInfo info)
        {
            if (info.Exists)
            {
                var attachment = new Attachment()
                {
                    Filename = info.Name,
                    ContentType = getContentType(info),
                    Body = getFileAsBase64ByteArray(info),
                    Size = (int)info.Length
                };
                return attachment;
            }

            throw new ArgumentException(string.Format("Hittar inte filen: {0}. Kontrollera att den inte är borttagen.", info.Name));
        }

        /// <summary>
        /// Gets a file as base64byteArray
        /// </summary>
        private byte[] getFileAsBase64ByteArray(FileInfo fileInfo)
        {
            byte[] base64encoded = new byte[0];
            if (fileInfo != null)
            {
                using (FileStream fs = new FileStream(fileInfo.FullName, FileMode.Open, FileAccess.Read))
                {
                    byte[] filebytes = new byte[fs.Length];
                    fs.Read(filebytes, 0, Convert.ToInt32(fs.Length));
                    base64encoded = filebytes;
                }
            }
            return base64encoded;
        }

        /// <summary>
        /// Gets the content type for an attachment.
        /// </summary>
        private string getContentType(FileInfo info)
        {
            foreach (AttachmentExtension extension in ConfigHelper.ConfigurationEntity.AttachmentExtensions)
            {
                if (info.Extension == extension.Extension)
                    return extension.ContentType;
            }

            throw new ArgumentException(string.Format("{0} : {1}",ResourceHelper.Resources[25].Text, info.Name));
        }
    }
}
